/*
 * Copyright © 2022 Soren Stoutner <soren@stoutner.com>.
 *
 * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
 *
 * Privacy Browser PC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Privacy Browser PC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
 */

// Application headers.
#include "PrivacyWebEngineView.h"
#include "databases/CookiesDatabase.h"
#include "windows/BrowserWindow.h"

// Construct the class.
PrivacyWebEngineView::PrivacyWebEngineView() : QWebEngineView(nullptr) {}

void PrivacyWebEngineView::addCookieToList(const QNetworkCookie &cookie) const
{
    //qDebug() << "Add cookie:  " << cookie.toRawForm();

    // Add the new cookie to the list.
    cookieListPointer->push_front(cookie);

    // Update the cookie if it is durable and has new data.
    if (CookiesDatabase::isUpdate(cookie))
        CookiesDatabase::updateCookie(cookie);

    // Update the cookies action.
    emit updateCookiesAction(cookieListPointer->size());
}

QWebEngineView* PrivacyWebEngineView::createWindow(QWebEnginePage::WebWindowType webWindowType) {
    // Get a handle for the browser window.
    BrowserWindow *browserWindowPointer = qobject_cast<BrowserWindow*>(window());

    // Create the requsted window type.
    switch (webWindowType) {
        case QWebEnginePage::WebBrowserTab: {
            // Create the new tab and return the privacy WebEngine view pointer.  `true` removes the focus from the blank URL line edit.
            // The new privacy WebEngine view pointer is returned so it can be populated with the link from the context menu.
            return browserWindowPointer->tabWidgetPointer->addTab(true);
        }

        case QWebEnginePage::WebBrowserWindow: {
            // Create a new browser window.
            BrowserWindow *newBrowserWindowPointer = new BrowserWindow();

            // Show the new browser window.
            newBrowserWindowPointer->show();

            // The new privacy WebEngine view pointer is returned so it can be populated with the link from the context menu.
            return newBrowserWindowPointer->tabWidgetPointer->loadBlankInitialWebsite();
        }

        default: {
            // Return an null pointer for opening a background tab and opening a web dialog.
            return nullptr;
        }
    }
}

void PrivacyWebEngineView::removeCookieFromList(const QNetworkCookie &cookie) const
{
    //qDebug() << "Remove cookie:  " << cookie.toRawForm();

    // Remove the cookie from the list.
    cookieListPointer->remove(cookie);

    // Update the cookies action.
    emit updateCookiesAction(cookieListPointer->size());
}
